/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the other or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.

Java Native Interface (JNI) for CryptoAPI (limited interface)
Author : CARDON de Lichtbuer Rodolphe
Juli 2006
rodolphe@wol.be
-------------------------
Limited Interface content
-------------------------
CertOpenStore
*/

#pragma comment (lib, "crypt32.lib")
#pragma comment (lib, "cryptui.lib")
#pragma comment (lib, "advapi32.lib")
#pragma comment (lib, "kernel32.lib")
#pragma comment (lib, "Cryptnet.lib")

#include "windows.h"
#include "include/be_cardon_nativecall_cryptoapi_CryptoAPICalls.h"
#include "jni.h"
#include "wincrypt.h"

static void throwCryptoAPIException(JNIEnv *env, const char *msg);


/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptAquireContext
 * Signature: (ILjava/lang/String;Ljava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptAquireContextW
  (JNIEnv *env, jobject obj, jstring StringContainer, jstring StringProvider, jint dwProvType, jint dwFlags){
		HCRYPTPROV* phProv = NULL;
		LPCWSTR pszContainer = (LPCWSTR) env->GetStringChars(StringContainer, 0);
		LPCWSTR pszProvider = (LPCWSTR) env->GetStringChars(StringProvider, 0);
		if(!CryptAcquireContextW(phProv, pszContainer,pszProvider, dwProvType, dwFlags)){
			const char* msg = "CryptAcquireContext failed";
			throwCryptoAPIException(env, msg);
		}

		env->ReleaseStringChars(StringContainer, (const jchar*)pszContainer);
	    env->ReleaseStringChars(StringProvider, (const jchar*)pszProvider);
		return (jint) phProv;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptReleaseContext
 * Signature: (I)Z
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptReleaseContext
  (JNIEnv *env, jobject obj, jint jhProv){
		if(!CryptReleaseContext(jhProv, 0)){
			const char* msg = "CryptReleaseContext failed";
			throwCryptoAPIException(env, msg);
		}
		return;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptGetUserKey
 * Signature: (II)I
 */
JNIEXPORT jint JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptGetUserKey
  (JNIEnv *env, jobject obj, jint hProv, jint dwKeySpec){
	  HCRYPTKEY* phUserKey=NULL;
		if(!CryptGetUserKey(hProv, dwKeySpec, phUserKey)){
			const char* msg = "CryptGetUserKey failed";
			throwCryptoAPIException(env, msg);
		}
		return (jint)phUserKey;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptDestoyKey
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptDestoyKey
  (JNIEnv *env, jobject obj, jint hKey){
	    if(!CryptDestroyKey(hKey)){
			const char* msg = "CryptDestoyKey failed";
			throwCryptoAPIException(env, msg);
		}
		return;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptGetKeyParam
 * Signature: (IIII)I
 */
JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptGetKeyParam
  (JNIEnv *env, jobject obj, jint hKey, jint dwParam){

		DWORD* pwDataLen = 0;
		//retrieves number of bytes (pwDataLen) needed for the buffer
		if(!CryptGetKeyParam(hKey,dwParam, NULL, pwDataLen, 0)){
			const char* msg = "CryptGetKeyParam (1) failed";
			throwCryptoAPIException(env, msg);
		}
		
		//memory allocation
		jbyteArray bytearray = env->NewByteArray(*pwDataLen);
		BYTE* pbData = (BYTE*) env->GetByteArrayElements(bytearray,0);
		
		//retrieves byte[]
		if(!CryptGetKeyParam(hKey,dwParam, pbData, pwDataLen, 0)){
			const char* msg = "CryptGetKeyParam (2) failed";
			throwCryptoAPIException(env, msg);
		}

		//frees memory
		env->ReleaseByteArrayElements(bytearray, (jbyte*) pbData, 0);

		return bytearray;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptDecrypt
 * Signature: (IIZI[B)[B
 */
JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptDecrypt
  (JNIEnv *env, jobject obj, jint hKey, jint hHash, jboolean Final, jint dwFlags, jbyteArray data){
	    
		BYTE* pbData = (BYTE*) env->GetByteArrayElements(data, 0);
		DWORD dwDataLen = env->GetArrayLength(data);
		DWORD* pdwDataLen = &dwDataLen;
		if(!CryptDecrypt(hKey, hHash, Final, dwFlags, pbData, pdwDataLen)){
			const char* msg = "CryptDecrypt failed";
			throwCryptoAPIException(env, msg);
		}
		jbyteArray output = env->NewByteArray(*pdwDataLen);
		env->SetByteArrayRegion(output, 0, *pdwDataLen, (jbyte*)pbData);
		env->ReleaseByteArrayElements(data, (jbyte*)pbData, JNI_ABORT); // frees memory, don't copy the content back
		return output;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptEncrypt
 * Signature: (IIZI[BI)[B
 */
JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptEncrypt
  (JNIEnv *env, jobject obj, jint hKey, jint hHash, jboolean Final, jint dwFlags, jbyteArray data, jint dwBufLen){

		BYTE* pbData = new BYTE[dwBufLen];
		DWORD dwDataLen = env->GetArrayLength(data);
		DWORD* pdwDataLen = &dwDataLen;		
		env->GetByteArrayRegion(data, 0, (jsize)dwDataLen, (jbyte*) pbData);
	  	if(!CryptEncrypt(hKey, hHash, Final, dwFlags, pbData, pdwDataLen, dwBufLen)){
			const char* msg = "CryptEncrypt failed";
			throwCryptoAPIException(env, msg);
		}
		jbyteArray output = env->NewByteArray(*pdwDataLen);
		env->SetByteArrayRegion(output, 0, *pdwDataLen, (jbyte*)pbData);
		delete[] pbData; // frees memory
		return output;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptCreateHash
 * Signature: (III)I
 */
JNIEXPORT jint JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptCreateHash
  (JNIEnv *env, jobject obj, jint hProv, jint Algid, jint hKey){
	  HCRYPTHASH* phHash = (HCRYPTHASH*)malloc(sizeof(HCRYPTHASH)); 
	  if(!CryptCreateHash(hProv, Algid, hKey, 0, phHash)){
			const char* msg = "CryptCreateHash failed";
			throwCryptoAPIException(env, msg);
	  }
	  return (jint)*phHash;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptDestroyHash
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptDestroyHash
  (JNIEnv *env, jobject obj, jint hHash){
	  if(!CryptDestroyHash(hHash)){
			const char* msg = "CryptDestroyHash failed";
			throwCryptoAPIException(env, msg);
	  }
	  return;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptSignHash
 * Signature: (IIII)[B
 */
JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptSignHash
  (JNIEnv *env, jobject obj, jint hHash, jint dwKeySpec, jint dwFlags){

	  DWORD dwSigLen = 0;
	  //retrieves the signature length
	  if(!CryptSignHash(hHash,dwKeySpec,NULL, dwFlags, NULL, &dwSigLen)){
			const char* msg = "CryptSignHash (1) failed";
			throwCryptoAPIException(env, msg);
			printf("on continue...\n");
			return NULL;
	  }

	  //mem alloc
	  jbyteArray bytearraySignature = env->NewByteArray(dwSigLen);
	  BYTE* pbSignature = (BYTE*) env->GetByteArrayElements(bytearraySignature,0);

	  //retrieves the signature
	  if(!CryptSignHash(hHash,dwKeySpec,NULL, dwFlags, pbSignature, &dwSigLen)){
			const char* msg = "CryptSignHash (1) failed";
			throwCryptoAPIException(env, msg);
			return NULL;
	  }

	  //frees the memory
	  env->ReleaseByteArrayElements(bytearraySignature, (jbyte*) pbSignature, 0);

	  return bytearraySignature;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptHashData
 * Signature: (I[BI)V
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptHashData
  (JNIEnv *env, jobject obj, jint hHash, jbyteArray data, jint dwFlags){
		
		BYTE* pbData = (BYTE*)env->GetByteArrayElements(data, NULL);
		DWORD dwDataLen = env->GetArrayLength(data);
		if(!CryptHashData(hHash, pbData, dwDataLen, dwFlags)){
		const char* msg = "CryptHashData failed";
			throwCryptoAPIException(env, msg);
		    return;
		}
		env->ReleaseByteArrayElements(data, (jbyte*)pbData, JNI_ABORT); //don't copy back
		return;
}


/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptSetHashParam
 * Signature: (III)V
 */
JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptSetHashParam
  (JNIEnv *env, jobject obj, jint hHash, jint dwParam, jbyteArray Data){

	  const BYTE* pbData = (const BYTE*) env->GetByteArrayElements(Data, 0);
	  if(!CryptSetHashParam(hHash,dwParam,pbData,0)){
			const char* msg = "CryptSetHashParam failed";
			throwCryptoAPIException(env, msg);
			return NULL;
	  }
	  env->ReleaseByteArrayElements(Data, (jbyte*) pbData, 0);
	  return Data;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptGetHashParam
 * Signature: (III)I
 */
JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptGetHashParam
  (JNIEnv *env, jobject obj, jint hHash, jint dwParam){

	  DWORD* pdwDataLen = NULL;
	  // retrieves the HashParam length
	  if(!CryptGetHashParam(hHash, dwParam, NULL, pdwDataLen, 0)){
			const char* msg = "CryptGetHashParam (1) failed";
			throwCryptoAPIException(env, msg);
			return NULL;
	  }
	  //mem alloc
	  jbyteArray bytearrayHashParam = env->NewByteArray(*pdwDataLen);
	  BYTE* pbData = (BYTE*) env->GetByteArrayElements(bytearrayHashParam,0);

	  //retrieves HashParam
	  if(!CryptGetHashParam(hHash, dwParam, pbData, pdwDataLen, 0)){
			const char* msg = "CryptGetHashParam (2) failed";
			throwCryptoAPIException(env, msg);
			return NULL;
	  }

	  //frees the memory
	  env->ReleaseByteArrayElements(bytearrayHashParam, (jbyte*) pbData, 0);

	  return bytearrayHashParam;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertCloseStore
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertCloseStore
  (JNIEnv *env, jobject obj, jint hCertStore, jint dwFlags){
	  if(!CertCloseStore(
		  (HCERTSTORE)hCertStore, 
		  dwFlags)){
			const char* msg = "CertCloseStore failed";
			throwCryptoAPIException(env, msg);
	  }
	  return;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertEnumCertificatesInStore
 * Signature: (II)I
 */
JNIEXPORT jint JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertEnumCertificatesInStore
  (JNIEnv *env, jobject obj, jint hCertStore, jint pPrevCertContext){

	 // we must self control the result (NULL) and call GetLastError

	  PCCERT_CONTEXT nexCertContext = CertEnumCertificatesInStore(
		  (HCERTSTORE) hCertStore, 
		  (PCCERT_CONTEXT) pPrevCertContext);
	  if(!nexCertContext){
		  if(GetLastError()!=CRYPT_E_NOT_FOUND){
			const char* msg = "CertEnumCertificatesInStore failed (error is not CRYPT_E_NOT_FOUND)";
			throwCryptoAPIException(env, msg);
		  }
	  }
	  return (jint)nexCertContext;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertOpenStore
 * Signature: (IIIII)I
 */
JNIEXPORT jint JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertOpenStore
  (JNIEnv *env,   jobject obj, jint lpszStoreProvider, jint dwMsgAndCertEncodingType, 
   jint hCryptProv, jint dwFlags, jint pvPara){
      HCERTSTORE certStore;
	  if(!(certStore = CertOpenStore( (LPCSTR) lpszStoreProvider, dwMsgAndCertEncodingType,
	                                     	hCryptProv, dwFlags,  (const void*) pvPara)))
		  {
				const char* msg = "CertOpenStore returned NULL";
				throwCryptoAPIException(env, msg);
				return NULL;
      }		  
	  return (jint) certStore;
}


/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertDuplicateCertificateContext
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertDuplicateCertificateContext
  (JNIEnv *env, jobject obj, jint pCertContext){
		PCCERT_CONTEXT duplCertContext = CertDuplicateCertificateContext((PCCERT_CONTEXT)pCertContext);
		return (jint) duplCertContext;
}



/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertFreeCertificateContext
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertFreeCertificateContext
  (JNIEnv *env, jobject obj, jint pCertContext){
	  CertFreeCertificateContext((PCCERT_CONTEXT)pCertContext);
	  return;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertGetNameStringW
 * Signature: (IIIII)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertGetNameStringW
  (JNIEnv *env, jobject obj, jint pCertContext, jint dwType, jint dwFlags, jint pvTypePara){

	  //check inputs to avoid ECXEPTION_ACCESS_VIOLATION 
	  if(!pCertContext){
		const char* msg = "CertGetNameStringW : argument pCertContext may not be NULL";
		throwCryptoAPIException(env, msg);
		return NULL;
	  }

	  //retrieves the string length (Unicode) = byte lenght/2 !! (1 unicode char = 2 bytes)
	  DWORD cchNameString = CertGetNameStringW((PCCERT_CONTEXT)pCertContext,dwType,dwFlags,(void*)pvTypePara,NULL,0);
	  if(cchNameString==1){
		//returned a null caracter
		return NULL;
	  }

	  if(cchNameString==0){
		//abnormal return !!
		const char* msg = "CertGetNameStringW returned an abnormal length of zero!";
		throwCryptoAPIException(env, msg);
		return NULL;
	  }

	  //alloc memory
	  byte* pszNameString = (byte*) new byte[cchNameString*2];
	  //retrieves the string name
	  DWORD size2 = CertGetNameStringW((PCCERT_CONTEXT)pCertContext,dwType,dwFlags,(void*)pvTypePara,(LPWSTR)pszNameString,cchNameString);

	  if(size2==1){
		//returned a null caracter
		return NULL;
	  }

	  jstring NameString = env->NewString((const jchar*) pszNameString, size2-1); // moins 1 pour supprimer le caractre NULL
	  delete[] pszNameString;
	  return NameString;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertVerifyRevocation
 * Signature: (IIIIIII)V
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertVerifyRevocation
  (JNIEnv *env, jobject obj, jint dwEncodingType, jint dwRevType, jint cContext, jint rgpvContext, jint dwFlags, jint pRevPara, jint pRevStatus){
		if(!CertVerifyRevocation(
			dwEncodingType,
			dwRevType,
			cContext,
			(PVOID*)rgpvContext,
			dwFlags,
			(PCERT_REVOCATION_PARA)pRevPara,
			(PCERT_REVOCATION_STATUS)pRevStatus)){
				const char* msg = "CertVerifyRevocation failed";
				throwCryptoAPIException(env, msg);
				return;
     	}
	    return;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptAcquireCertificatePrivateKey
 * Signature: (II)[I
 */
JNIEXPORT jintArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptAcquireCertificatePrivateKey
  (JNIEnv *env, jobject obj, jint pCert, jint dwFlags){

		HCRYPTPROV* phCryptProv = (HCRYPTPROV*)malloc(sizeof(HCRYPTPROV));
		DWORD* pdwKeySpec = (DWORD*)malloc(sizeof(DWORD));
		BOOL* pfCallerFreeProv = (BOOL*)malloc(sizeof(BOOL));
	    if(!CryptAcquireCertificatePrivateKey(
			(PCCERT_CONTEXT) pCert,
			dwFlags,
			NULL,
			phCryptProv,
			pdwKeySpec,
			pfCallerFreeProv)){
				const char* msg = "CryptAcquireCertificatePrivateKey failed";
				throwCryptoAPIException(env, msg);
				return NULL;
     	}
		jintArray returnValuesArray = env->NewIntArray(3);
		jint* returnVal = env->GetIntArrayElements(returnValuesArray,0);
		returnVal[0]= (jint) phCryptProv;
		returnVal[1]= (jint) pdwKeySpec;
		if(*pfCallerFreeProv){
			returnVal[2] = 1;
		}else{
			returnVal[2] = 0;
		}
		env->ReleaseIntArrayElements(returnValuesArray, returnVal, 0);
		return returnValuesArray;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptGetProvParam
 * Signature: (III)[B
 */
JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptGetProvParam
  (JNIEnv *env, jobject obj, jint hProv, jint dwParam, jint dwFlags){

	  //retrieves the length
	  DWORD* pdwDataLen = NULL;
	  if(!CryptGetProvParam(
		  (HCRYPTPROV)hProv,
		  dwParam,
		  NULL,
		  pdwDataLen,
		  dwFlags)){
				const char* msg = "CryptGetProvParam (1) failed";
				throwCryptoAPIException(env, msg);		
				return NULL;
	  }

	  //mem alloc
	  jbyteArray bytearrayProvParam = env->NewByteArray(*pdwDataLen);
	  BYTE* pbData = (BYTE*) env->GetByteArrayElements(bytearrayProvParam,0);

	  //retrieves ProvParam
	  if(!CryptGetProvParam(
		  (HCRYPTPROV)hProv,
		  dwParam,
		  pbData,
		  pdwDataLen,
		  dwFlags)){
				const char* msg = "CryptGetProvParam (2) failed";
				throwCryptoAPIException(env, msg);	
				return NULL;
	  }

	  //frees the memory
	  env->ReleaseByteArrayElements(bytearrayProvParam, (jbyte*) pbData, 0);

	  return bytearrayProvParam;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptSetProvParam
 * Signature: (IIII)V
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptSetProvParam
  (JNIEnv *env, jobject obj, jint hProv, jint dwParam, jint pbData, jint dwFlags){

	  if(!CryptSetProvParam(
		  (HCRYPTPROV)hProv,
		  dwParam,
		  (const BYTE*)pbData,
		  dwFlags)){
				const char* msg = "CryptSetProvParam failed";
				throwCryptoAPIException(env, msg);	
				return;
	  }
}


/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertGetCertificateContextProperty
 * Signature: (II[BI)[B
 */
JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertGetCertificateContextProperty
  (JNIEnv *env, jobject obj, jint pCertContext, jint dwPropId){

		DWORD pcbData = NULL;
		// retrives length
        if(!CertGetCertificateContextProperty(
			(PCCERT_CONTEXT) pCertContext,
			dwPropId,
			NULL,
			&pcbData)){
 				const char* msg = "CertGetCertificateContextProperty (1) failed";
				throwCryptoAPIException(env, msg);	
				return NULL;
	  }
	  //mem alloc
	  jbyteArray bytearrayCertificateContextProperty = env->NewByteArray(pcbData);
	  void* pvData = (void*) env->GetByteArrayElements(bytearrayCertificateContextProperty,0);
      //retrieves CertificateContextProperty
      if(!CertGetCertificateContextProperty(
			(PCCERT_CONTEXT) pCertContext,
			dwPropId,
			pvData,
			&pcbData)){
 				const char* msg = "CertGetCertificateContextProperty (2) failed";
				throwCryptoAPIException(env, msg);	
				return NULL;
	  }
	  
	  //frees the memory
	  env->ReleaseByteArrayElements(bytearrayCertificateContextProperty, (jbyte*) pvData, 0);
	  
	  return bytearrayCertificateContextProperty;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CryptRetrieveObjectByUrl
 * Signature: (Ljava/lang/String;Ljava/lang/String;IIIIII)I
 */
JNIEXPORT jint JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CryptRetrieveObjectByUrlW
  (JNIEnv *env, jobject obj, jstring Url, jstring ObjectOid, jint dwRetrievalFlags, jint dwTimeout,
   jint pCredentials, jint pvVerify, jint pAuxInfo){
       
	   LPCWSTR pszUrl = (LPCWSTR) env->GetStringChars(Url, 0);
	   LPCSTR pszObjectOid = (LPCSTR) env->GetStringUTFChars(ObjectOid, 0);
	   LPVOID* ppvObject = NULL;
	   if(!CryptRetrieveObjectByUrlW(
		    pszUrl,
		    pszObjectOid,
		    dwRetrievalFlags,
		    dwTimeout,
		    ppvObject,
		    NULL,
			(PCRYPT_CREDENTIALS)pCredentials,
		    (LPVOID)pvVerify,
		    (PCRYPT_RETRIEVE_AUX_INFO)pAuxInfo)){
 				const char* msg = "CryptRetrieveObjectByUrl failed";
				throwCryptoAPIException(env, msg);	
				return NULL;
	    }


		env->ReleaseStringChars(Url, (const jchar*)pszUrl);
		env->ReleaseStringUTFChars(ObjectOid, (const char*)pszObjectOid);

		return (jint)ppvObject;
}

JNIEXPORT jstring JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertNameToStrW
  (JNIEnv *env, jobject obj, jint dwCertEncodingType, jint pName, jint dwStrType){
	  
	  DWORD size; // size in character ! = 2 bytes!
	  DWORD size2;
	  CERT_NAME_BLOB Name = *(PCERT_NAME_BLOB)pName;
	  size=CertNameToStrW(dwCertEncodingType, 
		  &Name, 
		  dwStrType,
		  NULL,
		  0);
	  if (size==1){
 				const char* msg = "CertNameToStrW (1) returned an empty string";
				throwCryptoAPIException(env, msg);	
				return NULL;
	  }

	  LPWSTR psz = (LPWSTR) malloc(size*sizeof(WCHAR));
	  size2=CertNameToStrW(dwCertEncodingType, 
		  &Name, 
		  dwStrType,
		  psz,
		  size);
	  if(size2==1){
 				const char* msg = "CertNameToStrW (2) returned an empty string";
				throwCryptoAPIException(env, msg);	
				return NULL;
	  }
	  jstring str = env->NewString((const jchar *)psz,(size2-1)); //moins 1 pour supprimer la caractre NULL
	  free(psz);
	  return str;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertGetCertificateChain
 * Signature: (IIIIII)I
 */
JNIEXPORT jint JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertGetCertificateChain
  (JNIEnv *env, jobject obj, jint hChainEngine, jint pCertContext, jint pTime,
    jint hAdditionalStore, jint pChainPara, jint dwFlags){

	PCCERT_CHAIN_CONTEXT pChainContext;
    if(!CertGetCertificateChain(
		  (HCERTCHAINENGINE) hChainEngine,
		  (PCCERT_CONTEXT) pCertContext,
		  (LPFILETIME) pTime,
		  (HCERTSTORE) hAdditionalStore,
		  (PCERT_CHAIN_PARA) pChainPara,
		  (DWORD) dwFlags,
		  NULL,
		  &pChainContext)){
 				const char* msg = "CertGetCertificateChain failed";
				throwCryptoAPIException(env, msg);	
				return NULL;
	}
	return (jint)pChainContext;
}

/*
 * Class:     be_cardon_nativecall_cryptoapi_CryptoAPICalls
 * Method:    CertFreeCertificateChain
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_be_cardon_nativecall_cryptoapi_CryptoAPICalls_CertFreeCertificateChain
  (JNIEnv *env, jobject obj, jint pChainContext){
	   CertFreeCertificateChain((PCCERT_CHAIN_CONTEXT) pChainContext);
	   return;
}

//throw java Exception
static void throwCryptoAPIException(JNIEnv *env, const char *msg){
		  DWORD lasterr = GetLastError();
		  //printf("Native : GetLastError() returns : %d\n",lasterr);
		  

		  //last error number
		  jclass GetLastErrorClass = env->FindClass("be/cardon/nativecall/LastError");
   		  if (GetLastErrorClass==0){
				//printf(err, "unable to find the newGetLastError class, give up.");
				return;
			}
		  jmethodID mid =  env->GetStaticMethodID(GetLastErrorClass, "setLastError" , "(I)V");
		  if (mid==0) {
			  return;
		  }
		  env->CallStaticVoidMethod(GetLastErrorClass, mid, lasterr);


		  // throws error
		  jclass newExceptionClass = env->FindClass("be/cardon/nativecall/cryptoapi/CryptoAPIException");

		  if (newExceptionClass==0){
				//printf(err, "unable to find the new exception class, give up.");
				return;
			}

		  
		  env->ThrowNew(newExceptionClass, msg);

}
	