/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the other or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "jni.h"
#include "include/be_cardon_nativecall_LowLevelCalls.h"

#define DEFAULT_MAX_CHARS_NUMBER_IN_UNICODE_STRING  1000

static void throwNativeCallException(JNIEnv *env, const char *msg);
static void* memoryAlloc(JNIEnv *env, int size);
static int getUnicodeLength(JNIEnv *env, char* unicodeString, int maxCharNumber);


JNIEXPORT jint JNICALL Java_be_cardon_nativecall_LowLevelCalls_readInt
  (JNIEnv *env, jobject obj, jint address){
	  // to do : if address = 0 => throw exception...
	//printf("readInt : %d",*(jint*)address);
	return *(jint*)address;
}

JNIEXPORT void JNICALL Java_be_cardon_nativecall_LowLevelCalls_writeInt
  (JNIEnv *env, jobject obj, jint address, jint value){
	jint *val = (jint*)address;
	*val = value;
	return;
}


JNIEXPORT jshort JNICALL Java_be_cardon_nativecall_LowLevelCalls_readShort
  (JNIEnv *env, jobject obj, jint ptShort){
	  return *(jshort*)ptShort;
}

JNIEXPORT jstring JNICALL Java_be_cardon_nativecall_LowLevelCalls_readUTFString
  (JNIEnv *env, jobject obj, jint ptStringUTF){
		return env->NewStringUTF((const char*)ptStringUTF);
}


JNIEXPORT jstring JNICALL Java_be_cardon_nativecall_LowLevelCalls_readUnicodeString
  (JNIEnv *env, jobject obj, jint ptStringUNICODE){
		int charLen = getUnicodeLength(env, (char*)ptStringUNICODE,DEFAULT_MAX_CHARS_NUMBER_IN_UNICODE_STRING);
        return env->NewString((const jchar*)ptStringUNICODE, charLen);
}



JNIEXPORT jboolean JNICALL Java_be_cardon_nativecall_LowLevelCalls_readBoolean
  (JNIEnv *env, jobject obj, jint ptBoolean){
		return *(jboolean*)ptBoolean;
}

JNIEXPORT void JNICALL Java_be_cardon_nativecall_LowLevelCalls_writeBoolean
  (JNIEnv *env, jobject obj, jint ptBoolean, jboolean boolValue){
		jboolean* boolPointer;
		boolPointer = (jboolean*)ptBoolean;
		*boolPointer = boolValue;
		return;
}


JNIEXPORT jbyteArray JNICALL Java_be_cardon_nativecall_LowLevelCalls_readBytes
  (JNIEnv *env, jobject obj, jint pointer, jint size){
	  jbyteArray array = env->NewByteArray(size);
	  env->SetByteArrayRegion(array, 0, size, (const jbyte*)pointer);
	  return array;
}

JNIEXPORT jint JNICALL Java_be_cardon_nativecall_LowLevelCalls_allocAndCopyBytes
  (JNIEnv *env, jobject obj, jbyteArray array){
	   jsize size = env->GetArrayLength(array);
	   void* pointer;
	   if(!(pointer=memoryAlloc(env, size))){
		   return NULL;
	   }
	   env->GetByteArrayRegion(array, 0, size, (jbyte*)pointer);
	   return (jint)pointer;
}

JNIEXPORT jint JNICALL Java_be_cardon_nativecall_LowLevelCalls_allocBytes
  (JNIEnv *env, jobject obj, jint size){
	   void* pointer;
	   if(!(pointer=memoryAlloc(env, size))){
		   return NULL;
	   
	   }return (jint)pointer;
}

JNIEXPORT void JNICALL Java_be_cardon_nativecall_LowLevelCalls_deleteAllocatedData
  (JNIEnv *env, jobject obj, jint address){
	  delete((void*)address);
	  return;
}

JNIEXPORT void JNICALL Java_be_cardon_nativecall_LowLevelCalls_writeBytes
  (JNIEnv *env, jobject obj, jint address, jbyteArray array){
	   jsize size = env->GetArrayLength(array);
	   env->GetByteArrayRegion(array, 0, size, (jbyte*)address);
}



//throw java Exception
static void throwNativeCallException(JNIEnv *env, const char *msg){
		 
		  // throws error
		  jclass newExceptionClass = env->FindClass("be/cardon/nativecall/NativeCallException");

		  if (newExceptionClass==0){
				//printf(err, "unable to find the new exception class, give up.");
				return;
			}
		  env->ThrowNew(newExceptionClass, msg);
}

//memory allocation and error handling
static void* memoryAlloc(JNIEnv *env, int size){
	void* pointer;
	if(!(pointer=malloc(size))){
 				const char* msg = "Memory allocation failed";
				throwNativeCallException(env, msg);	
				return NULL;
	}
	return pointer;
}

//get NULL terminated UNICODE string length (the NULL character is not counted).
static int getUnicodeLength(JNIEnv *env, char* unicodeString, int maxCharNumber){
	for(int CharCounter = 0; CharCounter < maxCharNumber; CharCounter++){
		int byteCount = CharCounter * 2;
		if (unicodeString[byteCount]==0 && unicodeString[byteCount+1]==0){
			return CharCounter;
		}
	}
	// > max
	const char* msg = "getUnicodeLength failed. Maximum maxCharNumber has been reached.";
	throwNativeCallException(env, msg);	
	return NULL;
}