/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package be.cardon.utils;

import java.io.*;
import java.net.*;

/**
 *
 * @author Rodolphe
 */
public class LibraryLoader {
    
    public static String tempDirPropertyName = "java.io.tmpdir";
    public static String libPathPropertyName = "java.library.path";
    public static String NOT_FOUND = "NOT_FOUND";
    
    /** Creates a new instance of LibraryLoader */
    public LibraryLoader() {
    }

    public static void loadLib(ClassLoader classLoader, String libName, String extension)
    throws IOException, InterruptedException{
        
        Debug.println("LibraryLoader: trying loading library: "+libName);
        
        if(classLoader==null){
            Debug.println("LibraryLoader: using SystemClassLoader");
            classLoader = ClassLoader.getSystemClassLoader();
        }

        String filename = libName + "." + extension;
        Debug.println("filename: "+filename);        
        URL inputStreamLibURL = classLoader.getResource(filename);
        if(inputStreamLibURL==null){
            throw new IOException("Resource not found: "+filename);
        }
        
        String tempPath = System.getProperty(tempDirPropertyName, NOT_FOUND);
        if(tempPath.equals(NOT_FOUND)){
            throw new IOException("Temporaty File not found");
        }
        File tempDir = new File(tempPath);
        
        //first try to overwrite the default file
        File defaultFile = new File(tempPath, filename);
        
        boolean useDefaultFile = false;
        if(defaultFile.exists()){
            try{
                useDefaultFile = defaultFile.delete();
                //return false if the library cannot be deleted (locked)
            }catch(Exception e){
                e.printStackTrace();
                useDefaultFile = false;
            }
        }else{
            useDefaultFile = true;
        }
        
        File tempFile;
        if(useDefaultFile){
            tempFile = defaultFile;
        }else{
            tempFile = File.createTempFile(libName,"."+extension, tempDir);
        }
        
        Debug.println("LibraryLoader: tempFile = "+tempFile);
        
        LibraryLoader.copy(inputStreamLibURL.openStream() ,tempFile, 0);
        
        Runtime.getRuntime().load(tempFile.getAbsolutePath());
        
        Debug.println("LibraryLoader: loaded successfull library: "+libName);
        
        
    }
/*
    public File getNewTemporyFile() throws IOException{
        //nombre maximum d'essai, sinon chec
        int max  = 1000;
 
        String tempDirectoryString = System.getProperty("java.io.tmpdir", "NOT_FOUND");
        if(tempDirectoryString.equals("NOT_FOUND")){
            throw new IOException("Temporary directory not found.");
        }
        File tempDirectory = new File(tempDirectoryString);
 
 
        int counter=0;
        File tempDir;
        while(c<max)
            counter++;
            //random number string
            String randomNumber = Double.toString(Math.random());
 
            String directory = "WebSignApplet"+randomNumber;
            //the OS is responsible to delete this directory...
 
            tempDir= new File(tempDirectory, directory);
            if(!tempDir.exists()){
                break;
            }
        File.create
        }
 
    };
 */
    
    
    public static void copy(InputStream src, File dest, int bufferSize) throws IOException{
        if(bufferSize<=0){
            bufferSize = 2000; //default bytebuffer
        }
        InputStream is = src;
        OutputStream os = new BufferedOutputStream(new FileOutputStream(dest));
        byte[] buffer = new byte[bufferSize];
        int c;
        while((c = is.read(buffer))!= -1){
            os.write(buffer, 0, c);
        }
        is.close();
        os.close();
        return;
    }
    
    
    
    public static void copy(File src, File dest, int bufferSize) throws IOException{
        if(bufferSize<=0){
            bufferSize = 2000; //default bytebuffer
        }
        InputStream is = new BufferedInputStream(new FileInputStream(src));
        OutputStream os = new BufferedOutputStream(new FileOutputStream(dest));
        byte[] buffer = new byte[bufferSize];
        int c;
        while((c = is.read(buffer))!= -1){
            os.write(buffer, 0, c);
        }
        is.close();
        os.close();
        return;
    }
    
}
