/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package test;

import java.security.*;
import java.security.cert.*;
import java.util.*;
import be.cardon.utils.*;
/**
 *
 * @author Rodolphe
 */
public class ProviderTest2 {
    



    /** Creates a new instance of ProviderTest2 */
    public ProviderTest2() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try{
            //install provider
            java.security.Security.addProvider(new be.cardon.cryptoapi.provider.CryptoAPIProvider());

            //view keystore
            KeyStore store = KeyStore.getInstance("CryptoAPI");
            store.load(null, null);
            for(Enumeration<String> aliases=store.aliases();
                  aliases.hasMoreElements();){
                String alias = aliases.nextElement();
                System.out.println("-----------------------------------------------------------------------------");
                System.out.println("alias : "+alias);
                System.out.println("subject : "+
                         ((X509Certificate)store.getCertificate(alias)).getSubjectDN().getName());
                java.security.cert.Certificate[] chainCerts = store.getCertificateChain(alias);
                for(int it=0;it<chainCerts.length;it++){
                    X509Certificate chainCert = (X509Certificate)chainCerts[it];
                    System.out.println("certificate chain num "+it+" ="+ chainCert.getSubjectX500Principal() );
                }
            }
            
            
            //key for signing
            String KeyAliasForSign = "C=ZA, O=Thawte Consulting (Pty) Ltd., CN=Thawte Personal Freemail Issuing CA/SN:1834E4FE3ABAE46736C718621FBFA4CD";
            //String KeyAliasForSign = "C=BE, CN=Citizen CA, OID.2.5.4.5=200501/SN:1000000000008B38FAD6E5288E1C69E0";
            //String KeyAliasForSign = "C=BE, CN=Citizen CA, OID.2.5.4.5=200501/SN:100000000000B8527E550AA30EA09B92";
            //String KeyAliasForSign = "C=US, S=UT, L=Salt Lake City, O=The USERTRUST Network, OU=http://www.usertrust.com, CN=UTN-USERFirst-Client Authentication and Email/SN:1335AB91507F83063409942B1DF7414E";
            System.out.println("KeyAlias exists ?" + store.containsAlias(KeyAliasForSign));
            PrivateKey myPrivKey = (PrivateKey)store.getKey(KeyAliasForSign, null);
            
            //data to sign
            String dataString2Sign = "abc";
            byte [] data2Sign = dataString2Sign.getBytes("UTF-8");
            
            //signing service
            Signature sig = Signature.getInstance("CryptoAPI-SHA1");
            sig.initSign(myPrivKey);
            sig.update(data2Sign);
            byte[] signature = sig.sign();
            
            System.out.println("Sig = "+HexUtils.ByteArray2Hex(signature));
            
        }catch(Exception e){
            e.printStackTrace();
        }
        
    }
    
}
