/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package test;

import be.cardon.cryptoapi.*;
import be.cardon.utils.HexUtils;
import java.util.Iterator;
import be.cardon.nativecall.cryptoapi.Constants;

import be.cardon.nativecall.*;
import be.cardon.nativecall.cryptoapi.StructCERT_CHAIN_CONTEXT;
import be.cardon.nativecall.cryptoapi.StructCERT_CHAIN_ELEMENT;
import be.cardon.nativecall.cryptoapi.StructCERT_SIMPLE_CHAIN;
import be.cardon.nativecall.cryptoapi.StructCERT_CONTEXT;


/**This test uses the "be.cardon.cryptoapi" package
 * that contains high level classes 
 * - to displays certificates in the 'my' store with private keys
 * - to sign an "abc" UTF-8 encoded string
 *
 * @author CARDON Rodolphe
 */
public class SignTestCryptoAPIHigh {
    
    /** Creates a new instance of SignTestCryptoAPI */
    public SignTestCryptoAPIHigh() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
      
       try{
           
       be.cardon.utils.Debug.setDebug(true);
       //displays the certificates with private keys
       CAPIStore mystore = CAPIStore.openMyStore();
       for(Iterator<CAPICertificate> it=mystore.iteratorWithPrivateKey();it.hasNext();){
           CAPICertificate cert = it.next();
           System.out.println("--------------------------------------");
           System.out.println("Issuer RDN : " + cert.getIssuerRDN());
           System.out.println("Issuer SN : "+ cert.getIssuerSerialNumber());
           System.out.println("avant");
           CAPICertificateChain chain= new CAPICertificateChain(cert);
           System.out.println("aprs");
           
           StructCERT_CHAIN_CONTEXT chainContext = chain.GetNativeStructChainContext();
           int nbOfChains = chainContext.cChain().get();
           System.out.println("nombre de chaine ="+nbOfChains);
           StructCERT_SIMPLE_CHAIN[] chains = chainContext.Chain();
           for(int counter = 0;counter < nbOfChains;counter++){
               System.out.println("++++++++++++++");
               StructCERT_SIMPLE_CHAIN simpleChain = chains[counter];
               System.out.println("ok");
               
               System.out.println("trust status: " +simpleChain.TrustStatus());
               System.out.println("number of certificates: " +simpleChain.cElement().get());
               int certCount = simpleChain.cElement().get();
               StructCERT_CHAIN_ELEMENT[] certelements = simpleChain.element();
               for(int CertCounter = 0;CertCounter < certCount; CertCounter++){
                   System.out.println("---------");
                   System.out.println("cert num "+CertCounter);
                   StructCERT_CONTEXT certcontext = certelements[CertCounter].certContext();
                   CAPICertificate certFromChain = new CAPICertificate(certcontext.getAddress());
                   System.out.println(certFromChain.getX509Certificate().getIssuerX500Principal());
               }
               
           }
       }

       //uses the first certificate for signing operation
       CAPICertificate cert4sign = mystore.iteratorWithPrivateKey().next();
       System.out.println("======================================================================");
       System.out.println("cert for signing Issuer RDN = "+ cert4sign.getIssuerRDN());
       System.out.println("cert for signing SN = "+cert4sign.getIssuerSerialNumber());
       
       //sign "abc"
       String string2sign = "abc";
       byte[] data = string2sign.getBytes("UTF-8");
       CAPIHash hash = cert4sign.getPrivateKey(null).createHash(Constants.CALG_SHA1);
       hash.hashData(data);
       byte[] signature = hash.signHash();
       
       //display signature
       System.out.println("Signature : "+HexUtils.ByteArray2Hex(signature));
       
       //gharbage
       System.gc();
       
      }catch (Exception e){
          e.printStackTrace();
      } 
       
    }
    
}
