/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package test;

import be.cardon.nativecall.LowLevelCalls;
import be.cardon.nativecall.cryptoapi.CryptoAPICalls;
import be.cardon.nativecall.cryptoapi.CryptoAPIException;
import be.cardon.nativecall.cryptoapi.Constants;
import be.cardon.nativecall.NativeUnicodeString;
import be.cardon.utils.HexUtils;
import be.cardon.utils.ByteArrayTools;
/**This test uses the "be.cardon.nativecall.cryptoapi" package
 * and the "be.cardon.nativecall" package with low level functions
 * to display certificates and sign "abc" UTF-8 string.
 *
 * @author Rodolphe
 */
public class SignTestCryptoAPILow {
    
    /**
     * Creates a new instance of SignTestCryptoAPIveryLow 
     */
    public SignTestCryptoAPILow() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
      
       try{
           
       String message2sign = "abc";
       byte[] data2sign = message2sign.getBytes("UTF-8");
       
       CryptoAPICalls CAPI = new CryptoAPICalls();
       Constants CONST = new Constants();
       LowLevelCalls LC = new LowLevelCalls();
       
       int store=CAPI.CertOpenStore(
               CONST.CERT_STORE_PROV_SYSTEM, 
               0, 
               0, 
               CONST.CERT_SYSTEM_STORE_CURRENT_USER, 
               NativeUnicodeString.newInstance("My").getAddress());
       System.out.println("store handle = "+store);
       int pcert = 0;
       boolean loopEnd = false;
       while(!loopEnd){
           pcert = CAPI.CertEnumCertificatesInStore(store,pcert);
           if(pcert!=0){
                String certname = CAPI.CertGetNameStringW(pcert, CONST.CERT_NAME_FRIENDLY_DISPLAY_TYPE, 0, 0);
                System.out.println("certname = "+certname);
           }else{
            loopEnd=true;
           }
        }
       
      //select the first certif
      int mycert = CAPI.CertEnumCertificatesInStore(store,0);
      System.out.println("selected certif :" + CAPI.CertGetNameStringW(mycert,CONST.CERT_NAME_FRIENDLY_DISPLAY_TYPE,0,0));
      
      //aquire privatekey
      int[] cryptAcqPriKeyReturn = CAPI.CryptAcquireCertificatePrivateKey(mycert,CONST.CRYPT_ACQUIRE_CACHE_FLAG);
      int hCryptProv = LC.readInt(cryptAcqPriKeyReturn[0]);
      int dwKeySpec = LC.readInt(cryptAcqPriKeyReturn[1]);
      boolean fCallerFreeProv = (cryptAcqPriKeyReturn[2]==1);
      
      //createhash
      System.out.println("Create hash");
      int hash = CAPI.CryptCreateHash(hCryptProv,CONST.CALG_SHA1, 0);
      
      //hashdata
      System.out.println("Hash data");
      CAPI.CryptHashData(hash,data2sign,0);
      
      //signhash (reverse byte from little endian to big endian)
      System.out.println("Sign hash");
      byte[] signature = ByteArrayTools.reverseByteArray(CAPI.CryptSignHash(hash,dwKeySpec, 0));
      
      //print signature
      System.out.println("Print sig");
      String hexsign = HexUtils.ByteArray2Hex(signature);
      System.out.println(hexsign);
      
      //destroy hash
      System.out.println("Destroy hash");
      CAPI.CryptDestroyHash(hash);
      
      //release key context
      System.out.println("Release context");
      CAPI.CryptReleaseContext(hCryptProv);
      
      //close store
      System.out.println("Close store");
      CAPI.CertCloseStore(store, 0);
             
      }catch (Exception e){
          e.printStackTrace();
      } 
       
    }
    
}
