/*
Copyright (c) 2006, CARDON DE LICHTBUER Rodolphe
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list 
of conditions and the following disclaimer. 

2. Redistributions in binary form must reproduce the above copyright notice, this 
list of conditions and the following disclaimer in the documentation and/or 
other materials provided with the distribution. 

3. The name of the author or contributors may not be used to endorse or promote 
products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
DAMAGE.
 */

package test;

import java.security.KeyStore;
import java.io.FileInputStream;
import java.security.KeyStoreException;
import java.util.Enumeration;
import java.util.Collection;
import java.util.Iterator;
import java.security.Signature;
import java.security.PrivateKey;
import java.security.PublicKey;
import be.cardon.utils.HexUtils;
import java.security.cert.CertificateFactory;
import java.security.cert.Certificate;

/**
 *
 * @author Rodolphe
 */
public class SignTestPKCS12 {
    
    /** Creates a new instance of SignTestPKCS12 */
    public SignTestPKCS12() {
    }
    
    /**
     * @param args the command line arguments
     */
    
    public static void main(String[] args) {
        try{
        String filename = "D:/Mes documents/- ERM/TFE crypto/certificats (divers  trier)/clef prive freemail protege avec mdp et cert.pfx";
        String filenameCert = "D:/Mes documents/- ERM/TFE crypto/certificats (divers  trier)/freemailDER.cer";
        String filenameCertPKCS7 = "D:/Mes documents/- ERM/TFE crypto/certificats (divers  trier)/freemailAllcert.p7b";
        String pass = be.cardon.utils.Keyboard.input("Password :");
        String signatureAlg = "SHA1withRSA";
        String messageToSign = "abc";
        
        //encode message
        byte[] data  = messageToSign.getBytes("UTF-8");
        System.out.println("data hex="+HexUtils.ByteArray2Hex(data));
        
        //list keys
        KeyStore pkcs12store = KeyStore.getInstance("pkcs12");
        pkcs12store.load(new FileInputStream(filename), pass.toCharArray());
        PrintContent(pkcs12store);
        
        
        //take the first alias
        String myalias=pkcs12store.aliases().nextElement();
        PrivateKey myPrivateKey = (PrivateKey)pkcs12store.getKey(myalias, pass.toCharArray());
        System.out.println("selected key alias ="+myalias);
        
        //sign
        Signature sign = Signature.getInstance(signatureAlg);
        sign.initSign(myPrivateKey);
        sign.update(data);
        byte[] thesignature = sign.sign();
        System.out.println("signature length in bytes ="+thesignature.length);
        System.out.println("signature in hex :"+ HexUtils.ByteArray2Hex(thesignature));
        
        //import certificate
        CertificateFactory cf = CertificateFactory.getInstance("X.509");
        Collection c = cf.generateCertificates(new FileInputStream(filenameCert));
        
        System.out.println("nombre de certificats= "+c.size());
        for(Iterator<Certificate> i = (Iterator<Certificate>) c.iterator();i.hasNext();) {
        Certificate cert = i.next();
        //System.out.println(cert);
        }
        
        Certificate mycert = (Certificate)c.iterator().next();
        PublicKey pubkey = mycert.getPublicKey();

        //verif
        Signature signVerif = Signature.getInstance(signatureAlg);
        signVerif.initVerify(pubkey);
        signVerif.update(data);
        if(signVerif.verify(thesignature)){
            System.out.println("Verification OK");
        }else{
            System.out.println("Verification ECHEC");
        }
        
        
        }catch(Exception e){
            e.printStackTrace();
        }
    }
 
    public static void PrintContent(KeyStore store) throws KeyStoreException{
        System.out.println("Content of store:");
        for(Enumeration<String> aliases=store.aliases(); aliases.hasMoreElements();){
            String alias = aliases.nextElement();
            if(store.isKeyEntry(alias)){
                System.out.println("Key alias="+alias);
            }else{
                System.out.println("Certificate alias="+alias);
            }
        }
    }
}
